use std::env;
use std::fs::File;
use std::io::Write;
use std::process;
use tectonic;

fn main() {
    let args: Vec<String> = env::args().collect();
    if args.len() != 2 {
        println!("Error: You need to run this program with exactly 1 argument");
        process::exit(1);
    }
    let equation = computorv1::parse(&args[1]).unwrap_or_else(|e| {
        println!("Error during parsing: {e}");
        process::exit(1);
    });
    println!("{:?}", equation);
    let is_equation = computorv1::maths::evaluator::is_equation(&equation);
    let evaluated = computorv1::maths::evaluator::evaluate(equation).unwrap_or_else(|e| {
        println!("Error during evaluation: {e}");
        process::exit(1);
    });
    computorv1::pretty(&evaluated);
    if is_equation {
        computorv1::maths::solver::solve(evaluated);
    }
/*
    let latex = r#"
    \documentclass{article}
    \begin{document}
    Hello, world!
    \end{document}
    "#;

    let pdf_data: Vec<u8> = tectonic::latex_to_pdf(latex).expect("processing failed");
    println!("Output PDF size is {} bytes", pdf_data.len());

     // Write the modified data.
     let mut f = File::create("file-modified.pdf").unwrap();
     f.write_all(pdf_data.as_slice()).unwrap();
     */
}
